//
// Author: 
//   Mikael Hallendal <micke@imendio.com>
//
// (C) 2004 Imendio HB
// 

using GConf;
using Glade;
using Gdk;
using Gtk;
using GtkSharp;
using Gnome;
using Mono.Posix;
using System;
using System.Collections;
using System.IO;
using System.Text;

namespace Imendio.Blam {
    
    public class Application : Program {
	public static Application TheApp;
	
	[Widget] Gtk.Window         mainWindow;
	[Widget] Gtk.ScrolledWindow channelListSw;
	[Widget] Gtk.ScrolledWindow itemListSw;
	[Widget] Gtk.ScrolledWindow itemViewSw; 
	[Widget] Gtk.Paned          channelPaned;
	[Widget] Gtk.Paned          itemPaned;
	[Widget] Gtk.Label          blogNameLabel;
	[Widget] Gtk.Statusbar      statusbar;
	[Widget] Gtk.HBox           labelSpinnerBox;
	[Widget] Gtk.Label          channelsLabel;
	private string              channelsLabelText;
	
	[Widget] Gtk.MenuItem       nextUnreadMenuItem;
	[Widget] Gtk.MenuItem       updateChannelMenuItem;
	[Widget] Gtk.MenuItem       removeChannelMenuItem;
	[Widget] Gtk.MenuItem       editChannelMenuItem;
	[Widget] Gtk.MenuItem       printMenuItem;

	private Gtk.FileSelection   exportFileDialog;
	
	private Spinner spinner;

	private ChannelList channelList;
	private ItemList    itemList;
	private ItemView    itemView;

	private Cache         cache;

	public static string BaseDir;
	
	private MessageConnection messageConn;

	private AddChannelDialog  addChannelDialog;
	private ChannelDialog     channelDialog;
	private PreferencesDialog preferencesDialog;
	private OpmlDialog        opmlDialog;
	
	private ChannelCollection mCollection;

	enum TargetType {
	    String,
	    UriList
	};

	public static TargetEntry[] DragEntries = new TargetEntry[] {
	    new TargetEntry("STRING", 0, (uint)TargetType.String),
	    new TargetEntry("text/plain", 0, (uint)TargetType.String),
	    new TargetEntry("text/uri-list", 0, (uint)TargetType.UriList)
	};

	public Gtk.Window Window {
	    get {
		return (Gtk.Window) mainWindow;
	    }
	}

	public ChannelCollection CCollection {
	    get {
		return mCollection;
	    }
	}
	
	public Application (string[] args, params object[] props) : base ("Blam", Defines.VERSION, Modules.UI, args, props)
	{
	    messageConn = new MessageConnection ("Blam");
	    if (!messageConn.IsServer) {
		messageConn.Send ("ShowWindow");
		messageConn.Close ();
		Gdk.Global.NotifyStartupComplete ();
		Environment.Exit (0);
	    }

	    messageConn.SetCallback (new MessageConnection.MessageReceivedHandler (MessageReceivedCb));

	    string homeDirectory = Environment.GetEnvironmentVariable("HOME");

	    Mono.Posix.Catalog.Init ("blam", Defines.GNOME_LOCALE_DIR);

	    BaseDir = homeDirectory + "/.gnome2/blam";
	    if (!File.Exists(BaseDir)) {
		Directory.CreateDirectory(BaseDir);
	    }

	    mCollection = ChannelCollection.LoadFromFile (BaseDir + "/collection.xml");

	    mCollection.ChannelUpdated         += ChannelUpdatedCb;
	    mCollection.ChannelAdded           += ChannelAddedCb;
	    mCollection.ChannelRemoved         += ChannelRemovedCb;
	    mCollection.ChannelRefreshStarted  += ChannelRefreshStartedCb;
	    mCollection.ChannelRefreshFinished += ChannelRefreshFinishedCb;

	    cache = new Cache(BaseDir);

	    PrepareGUI();

	    GLib.Idle.Add (new GLib.IdleHandler (IdleRefreshAll));
	}

	private void PrepareGUI()
	{
	    Glade.XML gladeXML = Glade.XML.FromAssembly("blam.glade", 
							"mainWindow", null);
	    gladeXML.Autoconnect(this);
					      
	    channelList = new ChannelList(mCollection.Channels);
	    channelListSw.Child = channelList;

	    channelList.ChannelSelectedEvent += new ChannelEventHandler(ChannelSelected);
	    channelList.EditChannelEvent += new ChannelEventHandler(EditChannelActivated);
	    channelList.RemoveChannelEvent += new ChannelEventHandler(RemoveChannelActivated);
	    channelList.UpdateChannelEvent += new ChannelEventHandler(UpdateChannelActivated);
	    
	    itemView = new ItemView(cache, mCollection);
	    itemViewSw.Child = itemView;
	    itemView.OnUrl += new OnUrlHandler(OnUrl);
	    itemView.ItemLoaded += new ItemView.ItemLoadedHandler(ItemLoadDone);

	    itemList = new ItemList(itemView);
	    itemListSw.Child = itemList;

	    itemList.ItemSelected += new ItemList.ItemSelectedHandler(ItemSelected);

	    spinner = new Spinner();
	    labelSpinnerBox.PackEnd(spinner, false, false, 0);

	    SizeGroup sizeGroup = new SizeGroup(SizeGroupMode.Vertical);
	    sizeGroup.AddWidget(channelsLabel);
	    sizeGroup.AddWidget(labelSpinnerBox);

	    channelsLabelText = channelsLabel.Text;
	    UpdateChannelsLabel();
	    
	    printMenuItem.Sensitive = false;
	    SensitizeChanneMenuItems(false);

	    // Setup drag-n-drop
	    Gtk.Drag.DestSet(mainWindow, DestDefaults.All, 
			     DragEntries, DragAction.Copy | DragAction.Move);
	    mainWindow.DragDataReceived += DragDataReceivedCb;

	    RestoreWindowState();

	    mainWindow.Icon = Gdk.Pixbuf.LoadFromResource("blam.png");
	    mainWindow.ShowAll();
	    
	    channelDialog = new ChannelDialog (this);
	    addChannelDialog = new AddChannelDialog (this);
	    preferencesDialog = new PreferencesDialog (this.Window);
	    opmlDialog = new OpmlDialog (this.Window);
	    opmlDialog.ChannelAdded += mCollection.Add;
	    opmlDialog.ImportFinished += OpmlImportFinished;
	}

	private void ChannelSelected(Channel channel)
	{
	    if (channel == null) {
		SensitizeChanneMenuItems(false);
		return;
	    }
	    
	    itemList.CurrentChannel = channel;
	    
	    blogNameLabel.Markup = "<b>" + channel.Name + "</b>";
	    mainWindow.Title = "BLAM! - " + channel.Name;
	    
	    SensitizeChanneMenuItems(true);
	}

	private void SensitizeChanneMenuItems(bool sensitive)
	{
	    updateChannelMenuItem.Sensitive = sensitive;
	    removeChannelMenuItem.Sensitive = sensitive;
	    editChannelMenuItem.Sensitive = sensitive;
	}
	
	private void ItemSelected(Imendio.Blam.Item item)
	{
	    spinner.Start();
	    itemView.CurrentItem = item;
	    printMenuItem.Sensitive = true;
	}

	private void NextUnreadActivated(object obj, EventArgs args)
	{
	    if (channelList.NextUnread()) {
		itemList.NextUnread();
	    }
	}
  
	private void MainWindowKeyPressed (object obj, KeyPressEventArgs args)
	{
	    switch (args.Event.Key) {
	    case (Gdk.Key.period):
	    case (Gdk.Key.bracketright):
		// Couldn't figure out how to get this to the menu item itself.
		nextUnreadMenuItem.Activate();
		break;
	    }
	}

	private void PrintActivated(object obj, EventArgs args)
	{ 
	    Blam.PrintDialog dialog = new Blam.PrintDialog(itemView);
	    dialog.Run();
	}

	private void ImportOpmlActivated (object obj, EventArgs args)
	{
	    opmlDialog.Show ();
	}

	private bool ShowFileExistsDialog (Gtk.Window parentWindow, string fileName)
	{
	    string str = String.Format (Catalog.GetString ("File {0} already exists"), fileName);
	    
	    string msg = Catalog.GetString ("Do you want to overwrite the file?");

	    Gtk.Dialog dialog = ConfirmationDialog.Create (parentWindow,
							   Catalog.GetString ("_Overwrite"),
							   str, msg);

	    int result = dialog.Run ();
	    dialog.Destroy ();

	    switch (result) {
	    case (int)ResponseType.Ok:
		return true;
	    }
	    
	    return false;
	}

	private void ExportOpmlActivated (object obj, EventArgs args)
	{
	    if (exportFileDialog == null) {
		exportFileDialog = new Gtk.FileSelection (Catalog.GetString ("Export to..."));
		exportFileDialog.Modal = true;
		exportFileDialog.ShowFileops = true;
		exportFileDialog.TransientFor = mainWindow;
	    }

	    bool finished = false;
	    bool write = false;
	    string fileName = "";

	    while (!finished) {
		int result = exportFileDialog.Run ();

		switch (result) {
		case (int)ResponseType.Ok:
		    fileName = exportFileDialog.Filename;
		    
		    if (!File.Exists (fileName)) {
			write = true;
			finished = true;
		    } else {
			write = ShowFileExistsDialog (exportFileDialog, fileName);
			if (write) {
			    finished = true;
			}
		    }
		    break;
		case (int)ResponseType.Cancel:
		    finished = true;
		    break;
		}
	    }
	    
	    exportFileDialog.Hide ();

	    if (write) {
		OpmlWriter.Write (mCollection, fileName);
	    }
	}

	private void OpmlImportFinished (string status)
	{
	    uint contextId = statusbar.GetContextId("status");
	   
	    if (status != null) {
		statusbar.Push(contextId, status);
	    }
	}

	private void QuitActivated(object obj, EventArgs args)
	{
	    SaveWindowState();
	    mainWindow.Hide();

	    mCollection.SaveToFile ();

	    Quit();
	}

	private void CopyActivated (object obj, EventArgs args)
	{
	    itemView.Copy ();
	}

	private void PreferencesActivated (object obj, EventArgs args)
	{
	    preferencesDialog.Show ();
	}

	private void AboutActivated (object obj, EventArgs args)
	{
	    About.Show (mainWindow);
	}
	
	private void AddChannelActivated (object obj, EventArgs args)
	{
	    addChannelDialog.Show ();
	}

	private void EditChannelActivated (Channel channel) 
	{
	    if (channel != null) {
		channelDialog.Show (channel);
	    }
	}
	private void EditChannelActivated(object obj, EventArgs args)
	{
	    Channel channel = channelList.GetSelectedChannel();
	    EditChannelActivated(channel);
	}

	private void RemoveChannelActivated(Channel channel) 
	{
	    if (channel != null) {
		RemoveChannelDialog.Show (mainWindow, mCollection, channel);
	    }
	}

	private void RemoveChannelActivated(object obj, EventArgs args)
	{
	    Channel channel = channelList.GetSelectedChannel();

	    RemoveChannelActivated(channel);
	}

	private void UpdateChannelActivated(Channel channel)
	{
	    if (channel != null) {
		mCollection.Refresh (channel);
	    }
	}
	private void UpdateChannelActivated(object obj, EventArgs args)
	{
	    Channel channel = channelList.GetSelectedChannel();

	    UpdateChannelActivated(channel);
	}
	
	private void UpdateAllChannelsActivated(object obj, EventArgs args)
	{
	    mCollection.RefreshAll ();
	}

	private void DragDataReceivedCb(object o, DragDataReceivedArgs args)
	{
	    SelectionData d = args.SelectionData;
	    
	    if (d.Length < 0 && d.Format != 8){
		Gtk.Drag.Finish(args.Context, false, false, args.Time);
		return;
	    }
	   
	    Gtk.Drag.Finish(args.Context, true, true, args.Time);

	    UTF8Encoding encoding = new UTF8Encoding( );
	    string text = encoding.GetString(d.Data);

	    addChannelDialog.Show (text);
	}
	
        private void OnUrl (object o, OnUrlArgs args)
	{ 
	    uint contextId = statusbar.GetContextId("on_url");
	   
	    statusbar.Pop(contextId);
	    if (args.Url != null) {
		statusbar.Push(contextId, args.Url);
	    }
	}

	private void ChannelAddedCb (Channel channel)
	{
	    channelList.Add (channel);
	}
	
	private void ChannelUpdatedCb (Channel channel)
	{
	    channelList.Updated (channel);
	    UpdateChannelsLabel ();
	}

	private void ChannelRemovedCb (Channel channel) 
	{
	    channelList.Remove (channel);
	}

	private void ChannelRefreshStartedCb (Channel channel)
	{
	    uint contextId = statusbar.GetContextId("update-status");
	    string statusString;

	    statusString = String.Format (Catalog.GetString ("Refreshing: {0}"), channel.Name);

	    statusbar.Push (contextId, statusString);
	}

	private void ChannelRefreshFinishedCb (Channel channel)
	{
	    uint contextId = statusbar.GetContextId("update-status");

	    statusbar.Pop (contextId);
	    channelList.Updated (channel);
	    UpdateChannelsLabel ();

	    if (channelList.GetSelectedChannel () == channel) {
		itemList.UpdateList ();
	    }
	}

	private bool IdleRefreshAll ()
	{
	    mCollection.RefreshAll ();
	    return false;
	}

	private void ItemLoadDone()
	{
	    spinner.Stop();
	}

	private void RestoreWindowState() 
	{
	    int width, height;

	    width = Conf.Get ("ui/main_window_width", 600);
	    height = Conf.Get("ui/main_window_height", 400);

	    mainWindow.Resize(width, height);

	    width = Conf.Get("ui/channel_list_width", 180);
	    channelPaned.Position = width;

	    height = Conf.Get("ui/item_list_height", 100);
	    itemPaned.Position = height;
	}

	private void SaveWindowState()
	{
	    int height, width;

	    mainWindow.GetSize(out width, out height);
	    
	    Conf.Set("ui/main_window_width", width);
	    Conf.Set("ui/main_window_height", height);

	    Conf.Set("ui/channel_list_width", channelPaned.Position);
	    Conf.Set("ui/item_list_height", itemPaned.Position);

	    Conf.Sync ();
	}

	private void UpdateChannelsLabel()
	{
	    channelsLabel.Markup = string.Format("<b>" + 
						 channelsLabelText + 
						 "</b>",
						 mCollection.NrOfUnreadItems);
	}

	private void MessageReceivedCb (string message, IntPtr userData)
	{
	    if (!message.Equals ("ShowWindow")) {
		return;
	    }

	    mainWindow.Present ();
	    mCollection.RefreshAll ();
	}

	public static void Main(string[] args) 
	{
	    TheApp = new Application (args);
	    TheApp.Run ();
	    // new Application(args).Run();
	}
    }
}
